#include <stdio.h>
#include <winsock2.h>

const char* header = "HTTP/1.1 200 OK\nServer: Apache/2.0.55 (Win32)\nAccept-Ranges: bytes\nKeep-Alive: timeout=1500, max=100\nConnection: Keep-Alive\nContent-Type: application/x-dom-event-stream\n\n";

char *read_line (char *buf, size_t length, FILE *f)
  /**** Read at most 'length'-1 characters from the file 'f' into
        'buf' and zero-terminate this character sequence. If the
        line contains more characters, discard the rest.
   */
{
  char *p;

  if (p = fgets (buf, length, f)) {
    size_t last = strlen (buf) - 1;

    if (buf[last] == '\n') {
      /**** Discard the trailing newline */
      buf[last] = '\0';
    } else {
      /**** There's no newline in the buffer, therefore there must be
            more characters on that line: discard them!
       */
      fscanf (f, "%*[^\n]");
      /**** And also discard the newline... */
      (void) fgetc (f);
    } /* end if */
  } /* end if */
  return p;
} /* end read_line */

void printUsage(char* executableName) {
	printf("%s\n", executableName);
	printf("Usage: %s [binding port] [IP address of an adapter to bind]\n\n", executableName);
	printf("\tIf there are no arguments, the port will default to 80\n");
	printf("\tand the adapter used will be 127.0.0.1\n\n");
	printf("\tSpecifying 1 argument will set the port but not the adapter\n\n");
	exit(0);
}

void main(int argc, char* arg[]) {

	char* IPaddress = 0;
	int portNumber;

	if (argc == 2 && (strcmp(arg[1], "-h") == 0 || strcmp(arg[1], "--help") == 0)) {
		printUsage(arg[0]);
	}

	if (argc == 1) {
		portNumber = 80;
	} else if (argc == 2) {
		portNumber = atoi(arg[1]);
	} else if (argc == 3) {
		IPaddress = arg[2];
		portNumber = atoi(arg[1]);
	} else {
		printUsage(arg[0]);
	}

	if ((IPaddress != 0 && inet_addr(IPaddress) == INADDR_NONE) || portNumber == 0) {
		printf("\nOne or both of the argument specified were not the right type, try again.\n\n");
		printUsage(arg[0]);
	}

	//----------------------
	// Initialize Winsock.
	WSADATA wsaData;
	int iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
	if (iResult != NO_ERROR)
		printf("Error at WSAStartup()\n");

	//----------------------
	// Create a SOCKET for listening for
	// incoming connection requests.
	SOCKET ListenSocket;
	ListenSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	//ListenSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

	if (ListenSocket == INVALID_SOCKET) {
		printf("Error at socket(): %ld\n", WSAGetLastError());
		WSACleanup();
		return;
	}

	//----------------------
	// The sockaddr_in structure specifies the address family,
	// IP address, and port for the socket that is being bound.
	sockaddr_in service;
	service.sin_family = AF_INET;
	service.sin_addr.s_addr = ((IPaddress == (char*)0) ? inet_addr("127.0.0.1") : inet_addr(IPaddress));
	//service.sin_addr.s_addr = inet_addr("127.0.0.1");
	service.sin_port = htons(portNumber);
	//service.sin_port = htons(71);


	if (bind( ListenSocket, (SOCKADDR*) &service, sizeof(service)) == SOCKET_ERROR) {
		printf("bind() failed.\n");
		closesocket(ListenSocket);
		return;
	}

	//----------------------
	// Listen for incoming connection requests.
	// on the created socket
	if (listen( ListenSocket, 1 ) == SOCKET_ERROR)
		printf("Error listening on socket.\n");

	//----------------------
	// Create a SOCKET for accepting incoming requests.
	SOCKET AcceptSocket;
	printf("Listening on adapter %s on port %d\n", ((IPaddress == (char*)0) ? "127.0.0.1" : IPaddress), portNumber);
	printf("Waiting for client to connect...\n");

	//----------------------
	// Accept the connection.

	AcceptSocket = SOCKET_ERROR;
	while( AcceptSocket == SOCKET_ERROR ) {
		AcceptSocket = accept( ListenSocket, NULL, NULL );
	}
	printf("Client connected.\n");
	ListenSocket = AcceptSocket;



	char buf[300] = {0};
	//receive a header from the client
	int errorCode = recv(ListenSocket, buf, 300, 0);


	if (errorCode == SOCKET_ERROR) {
		errorCode = WSAGetLastError();
		printf("Couldn't receive data, error: %d", errorCode);
		closesocket(ListenSocket);
		WSACleanup();
		return;
	}

	printf("Received data:\n----------------------\n%s\n----------------------\n", buf);

	//send an HTTP header to the client. mozilla expects this for some reason
	errorCode = send(ListenSocket, header, strlen(header), 0);

	char commandString[50] = {0};
	int itemNumber = 6;

	while (1) {
		printf_s("\n\nPossible commands are: BID (b), NEW ITEM (n), CLOSE ITEM (c), QUIT (q)\nCommand: ");
		char *commandStringP = read_line(commandString, 50, stdin);
		//switch on the command key
		if (commandStringP[0] == 'n') {//NEW ITEM command
			char title[50] = "";
			char description[100] = "";
			//float startingPrice = 0.0;
			char startingPrice[25] = "";
			char picURL[255] = "";

			printf("Title: ");
			char *titleP = read_line(title, 50, stdin);

			printf("Description: ");
			char *descriptionP = read_line(description, 100, stdin);


			printf("Price: ");
			char *startingPriceP = read_line(startingPrice, 25, stdin);

			printf("Picture URL: ");
			char *picURLP = read_line(picURL, 255, stdin);

			sprintf_s(buf,
				"Event: new item\nTarget: #tableManager\ndata: %d\ndata: %s\ndata: %s\ndata: 0\ndata: %s\ndata: %s\n\n",
				itemNumber++,
				titleP,
				descriptionP,
				startingPriceP,
				picURLP);
			send(ListenSocket, buf, strlen(buf), 0);
			printf("Sent:\n---------------------\n%s\n---------------------\n\0", buf);
		} else if (commandString[0] == 'b') {//BID command
			char itemNumber[10] = "";//getchar() - 48 - 1;//convert from ASCII and then switch from 1-indexing to 0
			printf("Item: ");
			char *itemNumberP = read_line(itemNumber, 10, stdin);
			char newPrice[25] = "";
			printf("Amount: ");
			char *newPriceP = read_line(newPrice, 25, stdin);
			sprintf_s(buf, "Event: update bid\nTarget: #tableManager\ndata: %s\ndata: %s\n\n", itemNumberP, newPriceP);

			send(ListenSocket, buf, strlen(buf), 0);
			printf("Sent:\n---------------------\n%s\n---------------------\n\0", buf);

		} else if (commandString[0] == 'c') {//CLOSE an item
			char itemNumber[10] = "";
			printf("Item: ");
			char *itemNumberP = read_line(itemNumber, 10, stdin);

			sprintf_s(buf, "Event: close item\nTarget: #tableManager\ndata: %s\n\n", itemNumberP);
			send(ListenSocket, buf, strlen(buf), 0);
			printf("Sent:\n---------------------\n%s\n---------------------\n\0", buf);
		} else if (commandString[0] == 'q') {
			break;
		}
	}
	WSACleanup();
	return;
}
